#
#	$Id: ConfigFile.py,v 1.4 2003/03/19 14:10:18 gilles Exp $
#
# All rights reserved. Written by Gilles Lenfant <glenfant@bigfoot.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__doc__ = """ConfigFile main module

$Id: ConfigFile.py,v 1.4 2003/03/19 14:10:18 gilles Exp $"""

__version__ = "$Revision: 1.4 $"[11:-2]

# Standard Python imports
import sys, os
from cStringIO import StringIO
import ConfigParser

# Zope imports
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import view_management_screens
from Globals import DTMLFile

class ConfigFile(SimpleItem):
    """A ConfigFile object is a Zope wrapper to a ConfigParser object
    (from standard Python distro)
    
    Attributes :
    
    o id and title: as usual

    o filename: the name of the config file

    o autorefresh: if file is to be parsed again at each access (depending on file date)

    o _filepath: the full path to the config file

    o _confdata: a ConfigParser object

    o _filedate: OS modif time of file at last parse"""

    meta_type = 'ConfigFile'

    manage_options = (
        {'label': 'Edit', 'action': 'manage_main'},
        {'label': 'View', 'action': 'manage_view'}
        ) + SimpleItem.manage_options


    security = ClassSecurityInfo()

    def __init__(self, id, title='', filename='', autorefresh=0):
        self.id = id
        self.title = title
        self.filename = filename
        self.autorefresh = autorefresh
        try:
            self._filepath = _getConfigFilePath(filename)
            self._filedate = os.path.getmtime(self._filepath)
        except:
            self._filepath = self._filedate = None
        self._confdata = ConfigParser.ConfigParser()
        return

    ##################
    # ZMI Management #
    ##################

    manage_editForm = DTMLFile('zmi/editForm', globals())
    manage_main = manage_editForm

    security.declareProtected(view_management_screens, 'manage_edit')
    def manage_edit(self, title='', filename='', autorefresh=0, REQUEST=None):
        """Update object data"""
        errors = []
        self.title = title.strip()
        self.filename = filename.strip()
        self.autorefresh = autorefresh
        try:
            self._filepath = _getConfigFilePath(filename)
            self._confdata = ConfigParser.ConfigParser()
            self._confdata.read(self._filepath)
            self._filedate = os.path.getmtime(self._filepath)
        except Exception, e:
            et = StringIO()
            aux = sys.stderr
            sys.stderr = et
            type, value, tb = sys.exc_info()
            sys.excepthook(type, value, tb)
            sys.stderr = aux
            errors.append(et.getvalue().replace('\n', '<br>'))
            et.close()
        if REQUEST is not None:
            return self.manage_editForm(self,REQUEST, manage_tabs_message='Content changed.', errors=errors)
        return

    manage_view = DTMLFile('zmi/viewForm', globals())

    security.declareProtected(view_management_screens, 'manage_sourceFile')
    def manage_sourceFile(self):
        """Provides a copy of the config file contents"""
        try:
            fp = open(self._filepath)
            contents = fp.read()
            fp.close()
            return contents.strip()
        except:
            return None

    ################
    # User methods #
    ################

    def __getattr__(self, name):
        """Provides confdata methods/attributes"""
        forbidden = ('add_section', 'read', 'readfp', 'set', 'write', 'remove_option', 'remove_section', 'optionxform')
        if name in forbidden:
            return dummyFunc
        if self.autorefresh:
            if self._filedate != os.path.getmtime(self._filepath):
                self._confdata = ConfigParser.ConfigParser()
                self._confdata.read(self._filepath)
                self._filedate = os.path.getmtime(self._filepath)
        return getattr(self._confdata, name)

#############
# Utilities #
#############

def dummyFunc(*args, **kw):
    """Acceps anything and does nothing"""
    return

def _getConfigFilePath(filename):
    """Makes and validates the absolute path for the file"""
    filepath = os.path.join(ZOPE_HOME, 'etc', filename)
    if os.path.isfile(filepath):
        return filepath
    else:
        raise IOError('File %s does not exist' % filepath)
    return

###########################
# Adding objects from ZMI #
###########################

addForm = DTMLFile('zmi/addForm', globals())

def addFunction(dispatcher, id, REQUEST=None):
    """
    Create a new ConfigFile and add it to myself
    """
    obj = ConfigFile(id)
    dispatcher.Destination()._setObject(id, obj)
    if REQUEST is not None:
        REQUEST.RESPONSE.redirect(dispatcher.absolute_url() + ('/%s/manage_main' % id))
    return


###########################
# Registering the Product #
###########################

def initialize(registrar):
    registrar.registerClass(
        instance_class=ConfigFile,
        constructors=(addForm, addFunction),
        icon='www/zmi_icon.gif')
    return
